package com.taover.bazhuayun.analysis.web.module.heartbeat;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSONObject;
import com.taover.bazhuayun.analysis.web.form.HeartbeatInstanceCreateForm;
import com.taover.bazhuayun.analysis.web.form.HeartbeatInstanceUpdateForm;
import com.taover.bazhuayun.analysis.web.module.heartbeat.bean.ClientInstance;
import com.taover.bazhuayun.analysis.web.module.heartbeat.bean.ClientRequest;
import com.taover.bazhuayun.analysis.web.module.heartbeat.bean.Instance;
import com.taover.bazhuayun.analysis.web.module.heartbeat.bean.ServerInstance;
import com.taover.bazhuayun.analysis.web.module.heartbeat.holder.ClientHolder;
import com.taover.bazhuayun.analysis.web.module.heartbeat.holder.ClientHolderImpl;
import com.taover.bazhuayun.analysis.web.module.heartbeat.holder.ServerHolder;
import com.taover.bazhuayun.analysis.web.module.heartbeat.holder.ServerHolderImpl;
import com.taover.bazhuayun.analysis.web.module.heartbeat.service.HeartbeatDeployService;
import com.taover.bazhuayun.analysis.web.module.heartbeat.service.HeartbeatReformService;
import com.taover.bazhuayun.analysis.web.repository.AnalysisHeartbeatInstanceEntity;
import com.taover.bazhuayun.analysis.web.repository.AnalysisHeartbeatInstanceRepository;
import com.taover.bazhuayun.analysis.web.repository.AnalysisHeartbeatLogEntity;
import com.taover.bazhuayun.analysis.web.repository.AnalysisHeartbeatLogRepository;
import com.taover.repository.exception.MultiRowException;
import com.taover.repository.exception.NotFoundException;
import com.taover.util.UtilLog;
import com.taover.util.bean.ResultInfo;
import com.taover.util.bean.UtilResultInfo;

@Service
public class HeartbeatManagerImpl implements HeartbeatManager {
	//客户端holder
	private static ClientHolder clientHolder = new ClientHolderImpl();
	//服务端holder
	private static ServerHolder serverHolder = new ServerHolderImpl();
	
	@Resource
	private HeartbeatReformService heartbeatReformService;
	@Resource
	private HeartbeatDeployService heartbeatDeployService;
	@Resource
	private AnalysisHeartbeatInstanceRepository analysisHeartbeatInstanceRepository;
	@Resource
	private AnalysisHeartbeatLogRepository analysisHeartbeatLogRepository;
		
	/**
	 * 处理客户端心跳请求
	 * @param clientRequest
	 * @return
	 * @throws  
	 */
	@Override
	public ResultInfo dealClientRequest(HttpServletRequest request) {
		ClientRequest cRequest = ClientRequest.createClientRequest(request);
		if(!clientHolder.containInstanceCode(cRequest.getCode())) {
			return UtilResultInfo.getFailure("code="+cRequest.getCode()+"找不到对应的监控实例");
		}
		
		try {
			AnalysisHeartbeatInstanceEntity instance = this.analysisHeartbeatInstanceRepository.findEntityBySql("code='"+cRequest.getCode()+"' and role_type="+AnalysisHeartbeatInstanceRepository.ROLE_TYPE_CLIENT+" and status="+AnalysisHeartbeatInstanceRepository.STATUS_ENABLE, null);
			ResultInfo result = null;
			try {
				//刷新client信息
				clientHolder.dealClientRequest(cRequest, instance);
				result = UtilResultInfo.getSuccess("");
			}catch (Exception e) {
				result = UtilResultInfo.getFailure(e.getMessage());
			}
			
			//记录请求日志
			this.createLog(cRequest, request.getParameterMap(), result);
			
			return result;
		}catch (NotFoundException | MultiRowException e) {
			return UtilResultInfo.getFailure("查找code="+cRequest.getCode()+"对应的监控实例异常，异常信息："+e.getMessage());
		}		
	}
	
	private void createLog(ClientRequest cRequest, Map<String, String[]> data, ResultInfo response) {
		try {
			AnalysisHeartbeatLogEntity log = new AnalysisHeartbeatLogEntity();
			log.setClientIp(cRequest.getIp());
			log.setInstanceCode(cRequest.getCode());
			log.setResponse(JSONObject.toJSONString(response));
			log.setRequest(JSONObject.toJSONString(data));
			this.analysisHeartbeatLogRepository.addEntity(log, null);			
		}catch (Exception e) {
			UtilLog.error("数据库异常：客户端心跳请求，日志记录出现异常", e, HeartbeatManagerImpl.class);
		}
	}

	/**
	 * 刷新客户端状态
	 */
	@Override
	public void flushClientStatus() {
		List<ClientInstance> instances = clientHolder.getClientInstanceList();
		for(ClientInstance item: instances) {
			try {
				item.flush();
				if(item.needReform()) {
					this.heartbeatReformService.sendReform(item);
				}
				if(item.needDeploy()) {
					this.heartbeatDeployService.sendDeploy(item);
				}
			}catch (Exception e) {
				UtilLog.error("刷新客户端状态出现异常，code["+item.getIdentity()+"]", e, this.getClass());
			}
		}
	}
	
	/**
	 * 向服务器发送心跳
	 */
	@Override
	public void sendServerHeartbeat() {
		List<ServerInstance> instances = serverHolder.getServerInstanceList();
		for(ServerInstance item: instances) {
			try {
				item.flush();
				if(item.isSendNewHeartbeat()) {
					this.createLog(item);
				}
				if(item.needReform()) {
					this.heartbeatReformService.sendReform(item);
				}
				if(item.needDeploy()) {
					this.heartbeatDeployService.sendDeploy(item);
				}
			}catch (Exception e) {
				UtilLog.error("向服务器发送心跳出现异常，code["+item.getIdentity()+"]", e, this.getClass());
			}
		}
	}

	private void createLog(ServerInstance item) {
		try {
			AnalysisHeartbeatLogEntity log = new AnalysisHeartbeatLogEntity();
			log.setServerUrl(item.getInstance().getUrl());
			log.setInstanceCode(item.getIdentity());
			if(item.getLatestServerResponse() == null) {
				log.setResponse("null");			
			}else {
				log.setResponse(item.getLatestServerResponse().toString());
			}
			this.analysisHeartbeatLogRepository.addEntity(log, null);
		}catch (Exception e) {
			UtilLog.error("数据库异常：向服务器发送心跳，日志记录出现异常", e, HeartbeatManagerImpl.class);
		}
	}

	@Override
	public void loadInstance(List<AnalysisHeartbeatInstanceEntity> instanceList) throws Exception{
		if(instanceList == null || instanceList.isEmpty()) {
			return;
		}		
		for(AnalysisHeartbeatInstanceEntity serverItem: instanceList) {
			if(serverItem.getCode() == null || serverItem.getCode().trim().equals("")) {
				continue;
			}
			if(serverItem.getFixRateSec() == null) {
				serverItem.setFixRateSec(Instance.DEFAULT_FIX_RATE_SEC);
			}
			if(serverItem.getMaxWaitSec() == null) {
				serverItem.setMaxWaitSec(Instance.DEFAULT_MAX_WAIT_SEC);
			}
			
			try {
				this.registryRuntimeInstance(serverItem);	
			}catch (Exception e) {
				UtilLog.error("注册监控实例出现异常，code["+serverItem.getCode()+"]", e, this.getClass());
			}
		}
	}

	private void registryRuntimeInstance(AnalysisHeartbeatInstanceEntity serverItem) throws Exception {
		if(AnalysisHeartbeatInstanceRepository.ROLE_TYPE_SERVER == serverItem.getRoleType().intValue()) {
			serverHolder.registryServer(serverItem);
		}else if(AnalysisHeartbeatInstanceRepository.ROLE_TYPE_CLIENT == serverItem.getRoleType().intValue()) {
			clientHolder.registryClient(serverItem);
		}else {
			throw new RuntimeException("不支持的实例角色，INSTANCE："+JSONObject.toJSONString(serverItem));
		}
	}
	
	private void unregistryRuntimeInstance(AnalysisHeartbeatInstanceEntity serverItem) throws Exception {
		if(AnalysisHeartbeatInstanceRepository.ROLE_TYPE_SERVER == serverItem.getRoleType().intValue()) {
			serverHolder.unregistryServer(serverItem.getCode());
		}else if(AnalysisHeartbeatInstanceRepository.ROLE_TYPE_CLIENT == serverItem.getRoleType().intValue()) {
			clientHolder.unregistryClient(serverItem.getCode());
		}else {
			throw new RuntimeException("不支持的实例角色，INSTANCE："+JSONObject.toJSONString(serverItem));
		}
	}

	@Override
	public void createInstance(HeartbeatInstanceCreateForm form) {
		form.trimByRegexS();
		this.checkCreateForm(form);
		AnalysisHeartbeatInstanceEntity instance = this.constructInstance(form);
		Number id = this.analysisHeartbeatInstanceRepository.addEntity(instance, null);
		try {
			this.registryRuntimeInstance(this.analysisHeartbeatInstanceRepository.findEntityByID(id.longValue(), null));	
		}catch (Exception e) {
			UtilLog.error("注册监控实例出现异常，code["+instance.getCode()+"]", e, this.getClass());
		}
	}

	private void checkCreateForm(HeartbeatInstanceCreateForm form) {
		if(form.getCode() == null) {
			throw new RuntimeException("未传入code");
		}
		if(form.getFixRateSec() == null) {
			throw new RuntimeException("未传入检查间隔（秒）");
		}
		if(form.getMaxWaitSec() == null) {
			throw new RuntimeException("未传入最大等待时间（秒）");
		}
		if(form.getRoleType() == null) {
			throw new RuntimeException("未传入实例类型");
		}
		
	}

	private AnalysisHeartbeatInstanceEntity constructInstance(HeartbeatInstanceCreateForm form) {
		AnalysisHeartbeatInstanceEntity instance = new AnalysisHeartbeatInstanceEntity();
		instance.setCode(form.getCode());
		if(form.getDeployAtLostNum() != null) {
			instance.setDeployAtLostNum(form.getDeployAtLostNum());
		}
		if(form.getDeployUrl() != null) {
			instance.setDeployUrl(form.getDeployUrl());
		}
		if(form.getFixRateSec() != null) {
			instance.setFixRateSec(form.getFixRateSec());
		}
		if(form.getMaxWaitSec() != null) {
			instance.setMaxWaitSec(form.getMaxWaitSec());
		}
		if(form.getReformEmail() != null) {
			instance.setReformEmail(form.getReformEmail());
		}
		if(form.getReformMaxTime() != null) {
			instance.setReformMaxTime(form.getReformMaxTime());
		}
		if(form.getReformPhone() != null) {
			instance.setReformPhone(form.getReformPhone());
		}
		if(form.getReformWxid() != null) {
			instance.setReformWxid(form.getReformWxid());
		}
		if(form.getRoleType() != null) {
			instance.setRoleType(form.getRoleType());
		}
		if(form.getStatus() != null) {
			instance.setStatus(form.getStatus());
		}
		if(form.getUrl() != null) {
			instance.setUrl(form.getUrl());
		}
		if(form.getToken() != null) {
			instance.setToken(form.getToken());
		}
		return instance;
	}

	@Override
	public void updateInstance(String code, HeartbeatInstanceUpdateForm form) {
		form.trimByRegexS();
		AnalysisHeartbeatInstanceEntity instance = null;
		try {
			instance = this.analysisHeartbeatInstanceRepository.findEntityBySql("code='"+code+"'", null);
		} catch (NotFoundException | MultiRowException e1) {
			throw new RuntimeException("code="+code+"对应的实例数据异常（未找到或存在多条重复记录），请技术人员核实");
		}
		this.analysisHeartbeatInstanceRepository.updateEntityById(form.getSQLUpdateList(), instance.getId(), null);
		try {
			instance = this.analysisHeartbeatInstanceRepository.findEntityByID(instance.getId(), null);
			if(AnalysisHeartbeatInstanceRepository.STATUS_DISABLE == instance.getStatus().intValue()) {
				this.unregistryRuntimeInstance(instance);				
			}else {
				this.registryRuntimeInstance(instance);				
			}
		}catch (Exception e) {
			UtilLog.error("注销监控实例出现异常，code["+instance.getCode()+"]", e, this.getClass());
		}
	}

	@Override
	public void disableInstanceByCode(String code) {
		AnalysisHeartbeatInstanceEntity instance = null;
		try {
			instance = this.analysisHeartbeatInstanceRepository.findEntityBySql("code='"+code+"'", null);
		} catch (NotFoundException | MultiRowException e1) {
			throw new RuntimeException("code="+code+"对应的实例数据异常（未找到或存在多条重复记录），请技术人员核实");
		}
		List<Object[]> updateList = new ArrayList<Object[]>(1);
		updateList.add(new Object[] {"status", AnalysisHeartbeatInstanceRepository.STATUS_DISABLE});
		this.analysisHeartbeatInstanceRepository.updateEntityById(updateList, instance.getId(), null);
		try {			
			this.unregistryRuntimeInstance(instance);
		}catch (Exception e) {
			UtilLog.error("注销监控实例出现异常，code["+instance.getCode()+"]", e, this.getClass());
		}
	}

	@Override
	public void enableInstanceByCode(String code) {
		AnalysisHeartbeatInstanceEntity instance = null;
		try {
			instance = this.analysisHeartbeatInstanceRepository.findEntityBySql("code='"+code+"'", null);
		} catch (NotFoundException | MultiRowException e1) {
			throw new RuntimeException("code="+code+"对应的实例数据异常（未找到或存在多条重复记录），请技术人员核实");
		}
		List<Object[]> updateList = new ArrayList<Object[]>(1);
		updateList.add(new Object[] {"status", AnalysisHeartbeatInstanceRepository.STATUS_ENABLE});
		this.analysisHeartbeatInstanceRepository.updateEntityById(updateList, instance.getId(), null);
		try {			
			this.registryRuntimeInstance(instance);
		}catch (Exception e) {
			UtilLog.error("注销监控实例出现异常，code["+instance.getCode()+"]", e, this.getClass());
		}
	}

	@Override
	public void deleteInstance(String code) {
		AnalysisHeartbeatInstanceEntity instance = null;
		try {
			instance = this.analysisHeartbeatInstanceRepository.findEntityBySql("code='"+code+"'", null);
		} catch (NotFoundException | MultiRowException e1) {
			throw new RuntimeException("code="+code+"对应的实例数据异常（未找到或存在多条重复记录），请技术人员核实");
		}
		this.disableInstanceByCode(instance.getCode());
		this.analysisHeartbeatInstanceRepository.deleteEntityByID(instance.getId(), null);
	}
}
