package com.taover.bazhuayun.analysis.web.module.heartbeat.bean;

import com.taover.bazhuayun.analysis.web.repository.AnalysisHeartbeatInstanceEntity;
import com.taover.util.UtilLog;

public class ClientInstance implements Instance{
	private AnalysisHeartbeatInstanceEntity instance;
	private ClientRequest latestClientRequest;
	private long currFlushServerUnixtimeBaseline = 0L;
	private long latestClientServerUnixtime = System.currentTimeMillis()/1000;
	private long latestDetectServerUnixtime = this.latestClientServerUnixtime;
	private int lostClientRequestCount = 0;
	private boolean detectNewLost = false;
	
	public ClientInstance(AnalysisHeartbeatInstanceEntity instance) {
		this.instance = instance;
	}

	@Override
	public void flush() {
		//还未收到请求，则无需刷新失败次数
		if(latestClientRequest == null) {
			return;
		}
		
		synchronized (this) {
			this.detectNewLost = false;
			this.currFlushServerUnixtimeBaseline = System.currentTimeMillis()/1000;
			//通过服务器时间，判断是否超时未收到客户端请求
			if(isNewLoopForFixRateSec()) {
				this.latestDetectServerUnixtime = this.currFlushServerUnixtimeBaseline;
				if(isOverdueFixAndMaxWaitTime()) {
					++this.lostClientRequestCount;
					this.detectNewLost = true;
					UtilLog.infoForMessage("检测到超时未收到请求，code["+this.getIdentity()+"]，丢包统计["+this.lostClientRequestCount+"]", ServerInstance.class);
				}
			}
		}
	}
	
	private boolean isOverdueFixAndMaxWaitTime() {
		//System.out.println("isOverdueFixAndMaxWaitTime:"+(this.currFlushServerUnixtimeBaseline-this.latestClientServerUnixtime)+" > "+(this.instance.getFixRateSec()+this.instance.getMaxWaitSec()));
		return (this.currFlushServerUnixtimeBaseline-this.latestClientServerUnixtime) > (this.instance.getFixRateSec()+this.instance.getMaxWaitSec());
	}

	private boolean isNewLoopForFixRateSec() {
		//System.out.println("isNewLoopForFixRateSec:"+(this.currFlushServerUnixtimeBaseline-this.latestDetectServerUnixtime)+" > "+this.instance.getFixRateSec());
		return ((this.currFlushServerUnixtimeBaseline-this.latestDetectServerUnixtime) / this.instance.getFixRateSec()) > 0;
	}

	@Override
	public boolean needReform() {
		return this.detectNewLost && this.lostClientRequestCount>=1 && this.lostClientRequestCount<=this.instance.getReformMaxTime();
	}
	
	@Override
	public boolean needDeploy() {
		return this.detectNewLost && this.lostClientRequestCount == this.instance.getDeployAtLostNum();
	}

	public int getLostClientRequestCount() {
		return lostClientRequestCount;
	}

	public ClientRequest getLastestClientRequest() {
		return latestClientRequest;
	}

	public AnalysisHeartbeatInstanceEntity getInstance() {
		return instance;
	}

	public void dealClientRequest(ClientRequest clientRequest) {
		//System.out.println("dealClientRequest:"+clientRequest.toString());
		
		if(this.instance.getFixRateSec() == null) {
			this.instance.setFixRateSec(Instance.DEFAULT_FIX_RATE_SEC);
		}
		if(this.instance.getMaxWaitSec() == null) {
			this.instance.setMaxWaitSec(Instance.DEFAULT_MAX_WAIT_SEC);
		}
		
		synchronized (this) {
			this.latestClientRequest = clientRequest;
			this.latestClientServerUnixtime = System.currentTimeMillis()/1000;
			this.lostClientRequestCount = 0;			
		}
	}

	@Override
	public String getIdentity() {
		return this.instance.getCode();
	}

	public long getLatestClientServerUnixtime() {
		return this.latestClientServerUnixtime;
	}
	
	public long getUnixtime() {
		if(this.latestClientRequest == null) {
			return this.latestClientServerUnixtime;
		}else {
			return this.latestClientRequest.getUnixtime();
		}
	}
}
