package com.taover.bazhuayun.analysis.web.module.heartbeat.bean;

import java.net.SocketTimeoutException;

import com.taover.bazhuayun.analysis.web.repository.AnalysisHeartbeatInstanceEntity;
import com.taover.util.UtilEncrypt;
import com.taover.util.UtilHttpByOkHttp;
import com.taover.util.UtilLog;

public class ServerInstance implements Instance {
	private AnalysisHeartbeatInstanceEntity instance;
	private ServerResponse latestServerResponse;
	private long latestRequestUnixtime = -1;
	private long serverUnixtimeBaseline = 0L;
	private int errorServerResponseCount = 0;
	private boolean detectNewErrorServerResponse;
	private boolean isSendNewHeartbeat;
	
	public ServerInstance(AnalysisHeartbeatInstanceEntity instance) throws Exception {
		super();
		if(!instance.getUrl().toLowerCase().startsWith("http")) {
			throw new Exception("目前只支持http协议的URL");
		}
		this.instance = instance;
	}

	@Override
	public void flush() {
		this.serverUnixtimeBaseline = System.currentTimeMillis()/1000;
		this.detectNewErrorServerResponse = false;
		this.isSendNewHeartbeat = false;
		
		//如果小于设置的发送时间间隔，则不发送请求
		if(!this.isNewLoopForFixRateSec()) {
			return;
		}
		
		//发送数据包，并更新结构数据
		try {
			this.latestRequestUnixtime = this.serverUnixtimeBaseline;
			this.isSendNewHeartbeat = true;
			UtilLog.infoForMessage("向code["+this.getIdentity()+"],url["+this.instance.getUrl()+"]发送心跳请求", ServerInstance.class);
			
			this.latestServerResponse = ServerResponse.createByJSONString(UtilHttpByOkHttp.sendGet(this.instance.getUrl(), null, this.instance.getMaxWaitSec()));
			if(this.latestServerResponse.isCodeOk()) {
				this.errorServerResponseCount = 0;
			}
		} catch(SocketTimeoutException timeError) {
			this.latestServerResponse = ServerResponse.createOverdue();
		} catch (Exception e) {
			this.latestServerResponse = ServerResponse.createException(e);
		}
		
		//判断响应是否正常
		if(this.latestServerResponse.isOverdue() || !this.latestServerResponse.isCodeOk()) {
			++this.errorServerResponseCount;
			this.detectNewErrorServerResponse = true;
		}
	}
	
	private String getUrlParams() {
		long time = this.serverUnixtimeBaseline;
		String seckey = UtilEncrypt.MD5Lower32(this.instance.getCode()+time+this.instance.getToken());
		return "?code="+this.instance.getCode()+"&fixRateSec="+this.instance.getFixRateSec()+"&maxWaitSec="+this.instance.getMaxWaitSec()+"&unixtime="+time+"&seckey="+seckey;
	}

	private boolean isNewLoopForFixRateSec() {
		//System.out.println("isNewLoopForFixRateSec:"+(this.serverUnixtimeBaseline-this.latestRequestUnixtime)+" > "+this.instance.getFixRateSec());
		return (this.serverUnixtimeBaseline-this.latestRequestUnixtime) > this.instance.getFixRateSec();
	}
	
	public AnalysisHeartbeatInstanceEntity getInstance() {
		return instance;
	}

	@Override
	public String getIdentity() {
		return this.instance.getCode();
	}

	@Override
	public boolean needDeploy() {		
		return this.detectNewErrorServerResponse && this.errorServerResponseCount == this.instance.getDeployAtLostNum();
	}
	
	@Override
	public boolean needReform() {
		return this.detectNewErrorServerResponse
				&& this.errorServerResponseCount>=1 
				&& this.errorServerResponseCount<=this.instance.getReformMaxTime();
	}
	
	public long getLatestRequestUnixtime() {
		return latestRequestUnixtime;
	}

	public ServerResponse getLatestServerResponse() {
		return latestServerResponse;
	}
	
	public int getErrorServerResponseCount() {
		return errorServerResponseCount;
	}

	public boolean isIncrErrorServerResponseCount() {
		return detectNewErrorServerResponse;
	}

	public boolean isSendNewHeartbeat() {
		return isSendNewHeartbeat;
	}
}
