package com.taover.bazhuayun.analysis.web.module.heartbeat.holder;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.apache.commons.lang.StringUtils;

import com.taover.bazhuayun.analysis.web.module.heartbeat.bean.ClientInstance;
import com.taover.bazhuayun.analysis.web.module.heartbeat.bean.ClientRequest;
import com.taover.bazhuayun.analysis.web.repository.AnalysisHeartbeatInstanceEntity;
import com.taover.util.UtilEncrypt;

public class ClientHolderImpl implements ClientHolder{
	Map<String, ClientInstance> clientMap = new ConcurrentHashMap<String, ClientInstance>();

	@Override
	public List<ClientInstance> getClientInstanceList() {
		return new ArrayList<ClientInstance>(clientMap.values());
	}

	@Override
	public void flushClientStatus(ClientInstance clientInstance) {
		clientInstance.flush();
	}

	@Override
	public void dealClientRequest(ClientRequest clientRequest, AnalysisHeartbeatInstanceEntity instance) {
		this.validClientRequest(clientRequest, instance.getToken());
		synchronized (this.clientMap) {
			ClientInstance client = null;
			if(clientMap.containsKey(clientRequest.getIdentity())) {
				client = clientMap.get(clientRequest.getIdentity());
				clientMap.put(client.getIdentity(), client);
			}else {
				throw new RuntimeException("找不到code="+clientRequest.getIdentity()+"的监控实例");
			}
			client.dealClientRequest(clientRequest);
		}
	}

	private void validClientRequest(ClientRequest clientRequest, String token) {
		long time = clientRequest.getUnixtime();
		String seckey = UtilEncrypt.MD5Lower32(clientRequest.getCode()+time+token);
		if(StringUtils.isBlank(clientRequest.getCode())) {
			throw new RuntimeException("未传入code信息");
		}
		if(!seckey.equals(clientRequest.getSeckey())) {
			throw new RuntimeException("秘钥信息检查失败");
		}
	}

	@Override
	public void registryClient(AnalysisHeartbeatInstanceEntity instance) {
		synchronized (this.clientMap) {
			ClientInstance client = new ClientInstance(instance);
			clientMap.put(client.getIdentity(), client);
		}
	}

	@Override
	public void unregistryClient(String code) {
		if(code != null) {
			clientMap.remove(code);
		}
	}

	@Override
	public boolean containInstanceCode(String code) {
		if(code == null) {
			return false;
		}
		return clientMap.containsKey(code);
	}
}
