package com.taover.bazhuayun.analysis.web.module.heartbeat.service;

import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;

import javax.annotation.Resource;

import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.taover.bazhuayun.analysis.web.module.heartbeat.bean.ClientInstance;
import com.taover.bazhuayun.analysis.web.module.heartbeat.bean.Instance;
import com.taover.bazhuayun.analysis.web.module.heartbeat.bean.ServerInstance;
import com.taover.bazhuayun.analysis.web.repository.AnalysisHeartbeatInstanceEntity;
import com.taover.bazhuayun.analysis.web.repository.AnalysisHeartbeatReformEntity;
import com.taover.bazhuayun.analysis.web.repository.AnalysisHeartbeatReformRepository;
import com.taover.util.UtilEmail;
import com.taover.util.UtilLog;
import com.taover.util.UtilWeixinMsg;

@Service
public class HeartbeatReformServiceImpl implements HeartbeatReformService {
	@Resource
	private AnalysisHeartbeatReformRepository analysisHeartbeatReformRepository;
	
	private void sendWarnWeixin(Instance instance, String weixinContent) {
		if(StringUtils.isBlank(instance.getInstance().getReformWxid())) {
			return;
		}		
		try {
			String[] wxidArr = instance.getInstance().getReformWxid().split(",");
			for(String weixinWxid: wxidArr) {
				if(weixinWxid != null && !weixinWxid.trim().equals("")) {
					UtilWeixinMsg.sendTextMessage(weixinWxid, weixinContent);
					this.createWeixinReformLog(instance, weixinWxid, weixinContent);
				}				
			}			
		} catch (Exception e) {
			UtilLog.infoForMessage("发送微信消息失败，消息内容："+weixinContent, this.getClass());
			UtilLog.errorForException(e, this.getClass());			
		}
	}

	private void createWeixinReformLog(Instance instance, String weixinWxid, String weixinContent) {
		AnalysisHeartbeatReformEntity reform = new AnalysisHeartbeatReformEntity();
		reform.setInstanceCode(instance.getIdentity());
		reform.setReformContent(weixinContent);
		reform.setReformInfo(weixinWxid);
		reform.setReformType(AnalysisHeartbeatReformRepository.REFORM_TYPE_WEIXIN);
		this.analysisHeartbeatReformRepository.addEntity(reform, null);
	}

	private String formatWeixin(ServerInstance instance) {
		String htmlContent = "";
		AnalysisHeartbeatInstanceEntity entity = instance.getInstance();
		if(instance.getLatestServerResponse() == null) {
			htmlContent += "  未收到服务器响应\n";
		}else {
			if(instance.getLatestServerResponse().isOverdue()) {
				htmlContent += "  CODE["+instance.getIdentity()+"]监控异常，向接口["+entity.getUrl()+"]发送请求，响应超时["+entity.getMaxWaitSec()+"s]，请及时确认服务是否正常运行\n";
			}else if(!instance.getLatestServerResponse().isCodeOk()) {
				htmlContent += "  CODE["+instance.getIdentity()+"]监控异常，向接口["+entity.getUrl()+"]发送请求，响应数据不正常（详细请查看邮件），请及时确认服务是否正常运行\n";
			}else {
				htmlContent += "  CODE["+instance.getIdentity()+"]监控异常， 向接口["+entity.getUrl()+"]发送请求，未知异常，请及时确认服务是否正常运行\n";
			}
		}		
		htmlContent += "  最后一次请求时间： "+new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date(instance.getLatestRequestUnixtime()*1000));
		return htmlContent;
	}

	private String formatWeixin(ClientInstance instance) {
		String htmlContent = "  超过"+instance.getInstance().getFixRateSec()+"秒未收到请求，请确认CODE["+instance.getIdentity()+"]对应的服务是否运行正常\n";
		htmlContent += "  客户端最后一次请求时间： "+new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date(instance.getLastestClientRequest().getUnixtime()*1000))+"\n";
		htmlContent += "  累计次数("+instance.getInstance().getReformMaxTime()+"次后不再报警)："+instance.getLostClientRequestCount();
		return htmlContent;
	}

	private void sendWarnEmail(Instance instance, String subject, String htmlContent) {
		if(StringUtils.isBlank(instance.getInstance().getReformEmail())) {
			return;
		}
		try {
			String[] emailArr = instance.getInstance().getReformEmail().split(",");
			if(emailArr.length == 0) {
				return;
			}
			UtilEmail.sendHtmlMail(Arrays.asList(emailArr), subject, htmlContent);
			this.createHtmlReformLog(instance, emailArr, subject, htmlContent);
		} catch (Exception e) {
			UtilLog.infoForMessage("发送邮件失败，消息内容：[标题："+subject+"],[正文："+htmlContent+"]", this.getClass());
			UtilLog.errorForException(e, this.getClass());			
		}
	}

	private void createHtmlReformLog(Instance instance, String[] emailArr, String subject, String htmlContent) {
		AnalysisHeartbeatReformEntity reform = new AnalysisHeartbeatReformEntity();
		reform.setInstanceCode(instance.getIdentity());
		JSONObject data = new JSONObject();
		data.put("subject", subject);
		data.put("content", htmlContent);
		reform.setReformContent(data.toJSONString());
		reform.setReformInfo(StringUtils.join(emailArr, ","));
		reform.setReformType(AnalysisHeartbeatReformRepository.REFORM_TYPE_EMAIL);
		this.analysisHeartbeatReformRepository.addEntity(reform, null);
	}

	private String formatHtml(ServerInstance instance) {
		String htmlContent = "";
		htmlContent += "<h4 style=\"color:red;\">";
		AnalysisHeartbeatInstanceEntity entity = instance.getInstance();
		if(instance.getLatestServerResponse() != null) {
			if(instance.getLatestServerResponse().isOverdue()) {
				htmlContent += "  向接口["+entity.getUrl()+"]发送请求，响应超时["+entity.getMaxWaitSec()+"s]，请及时确认服务是否正常运行";
			}else if(!instance.getLatestServerResponse().isCodeOk()) {
				htmlContent += "  向接口["+entity.getUrl()+"]发送请求，响应数据不正常（详细请查看邮件），请及时确认服务是否正常运行";
			}else {
				htmlContent += "未知异常";	
			}
		}else {
			htmlContent += "未记录最后一次响应记录";
		}
		htmlContent += "</h4>";
		htmlContent += "<ul>";
		htmlContent += "<li>CODE： "+instance.getIdentity()+"</li>";
		htmlContent += "<li>URL： "+entity.getUrl()+"</li>";
		htmlContent += "<li>请求频率： "+entity.getFixRateSec()+"秒/次</li>";
		htmlContent += "<li>请求最大等待时间： "+entity.getMaxWaitSec()+"秒</li>";
		htmlContent += "<li>客户端请求时间戳： "+new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date(instance.getLatestRequestUnixtime()*1000))+"</li>";
		htmlContent += "<li>最后一次收到的响应包： <span style=\"color:gray;\">"+JSON.toJSONString(instance.getLatestServerResponse())+"</span></li>";
		htmlContent += "</ul>";
		return htmlContent;
	}

	private String formatHtml(ClientInstance instance) {
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		AnalysisHeartbeatInstanceEntity entity = instance.getInstance();
		String htmlContent = "";
		htmlContent += "<h4 style=\"color:red;\">";
		htmlContent += "超过"+entity.getFixRateSec()+"秒未收到请求，请确认CODE["+instance.getIdentity()+"]对应的客户端是否运行正常<br />";
		htmlContent += "客户端最后一次请求时间： "+new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date(instance.getUnixtime()*1000));
		htmlContent += "</h4>";
		htmlContent += "<ul>";
		htmlContent += "<li>CODE： "+instance.getIdentity()+"</li>";
		//htmlContent += "<li>IP： "+instance.getIp()+"</li>";
		htmlContent += "<li>请求频率： "+entity.getFixRateSec()+"秒/次</li>";
		htmlContent += "<li>请求最大等待时间： "+entity.getMaxWaitSec()+"秒</li>";
		htmlContent += "<li>客户端请求时间戳： "+sdf.format(new Date(instance.getUnixtime()*1000))+"</li>";
		htmlContent += "<li>服务器接收时间： "+sdf.format(new Date(instance.getLatestClientServerUnixtime()*1000))+"</li>";
		htmlContent += "<li>最后一次收到的请求包： <span style=\"color:gray;\">"+JSON.toJSONString(instance.getLastestClientRequest())+"</span></li>";
		htmlContent += "<li>累计次数("+entity.getReformMaxTime()+"次后不再报警)："+instance.getLostClientRequestCount()+"</li>";
		htmlContent += "</ul>";
		return htmlContent;
	}

	private String[] formatNormalEmailInfo(Instance instance) {
		String subject = "报警-项目可用性监控--监控ID["+instance.getIdentity()+"]";
		String htmlContent = "<h2>监控ID："+instance.getIdentity()+"</h2>";		
		if(instance instanceof ClientInstance) {
			htmlContent += this.formatHtml((ClientInstance)instance);
		}else if(instance instanceof ServerInstance) {
			htmlContent += this.formatHtml((ServerInstance)instance);
		}else {
			try {
				htmlContent += "<p>"+JSONObject.toJSONString(instance)+"</p>";	
			}catch (Exception e) {
				htmlContent += "<span style=\"color:red;\">对象无法序列化打印，instance.toString() ==>> "+instance.toString()+"</span>";
			}
		}
		return new String[] {subject, htmlContent};
	}
	
	@Override
	public void sendReform(Instance item) {
		String[] emailData = this.formatNormalEmailInfo(item);
		this.sendWarnEmail(item, emailData[0], emailData[1]);
		
		String weixinContent = this.formatNormalWeixinContent(item);
		this.sendWarnWeixin(item, weixinContent);
	}

	private String formatNormalWeixinContent(Instance instance) {
		String weixinContent = "报警-项目可用性监控\n";
		if(instance instanceof ClientInstance) {
			weixinContent += this.formatWeixin((ClientInstance)instance);
		}else if(instance instanceof ServerInstance) {
			weixinContent += this.formatWeixin((ServerInstance)instance);
		}else {
			try {
				weixinContent += "  JSON序列化："+JSONObject.toJSONString(instance);	
			}catch (Exception e) {
				weixinContent += "  对象无法序列化打印，instance.toString() ==>> "+instance.toString();
			}
		}
		return weixinContent;
	}

	@Override
	public void sendReformDeploy(Instance item) {
		String[] emailData = this.formatDeployEmailInfo(item);
		this.sendWarnEmail(item, emailData[0], emailData[1]);
		
		String weixinContent = this.formatDelayWeixin(item);
		this.sendWarnWeixin(item, weixinContent);
	}

	private String formatDelayWeixin(Instance instance) {
		return "通知-项目可用性监控--自动发布任务\n"
			+ "  CODE["+instance.getIdentity()+"]对应的服务多次检查失败\n"
			+ "  系统自动调度执行发布任务，服务怠机原因，请及时核实解决~";
	}

	private String[] formatDeployEmailInfo(Instance instance) {
		String subject = "通知-项目可用性监控--执行发布任务["+instance.getIdentity()+"]";
		String htmlContent = "<h2>监控ID："+instance.getIdentity()+"</h2>";
		if(instance instanceof ClientInstance) {
			htmlContent += this.formatHtml((ClientInstance)instance);
		}else if(instance instanceof ServerInstance) {
			htmlContent += this.formatHtml((ServerInstance)instance);
		}else {
			try {
				htmlContent += "<p>"+JSONObject.toJSONString(instance)+"</p>";	
			}catch (Exception e) {
				htmlContent += "<span style=\"color:red;\">对象无法序列化打印，instance.toString() ==>> "+instance.toString()+"</span>";
			}
		}
		return new String[] {subject, htmlContent};
	}

	@Override
	public void sendErrorMsg(Instance item, String response) {
		String[] emailData = this.formatErrorEmailInfo(item, response);
		this.sendWarnEmail(item, emailData[0], emailData[1]);
		
		String weixinContent = this.formatErrorWeixinContent(item, response);
		this.sendWarnWeixin(item, weixinContent);
	}

	private String[] formatErrorEmailInfo(Instance instance, String response) {
		String subject = "异常-项目可用监控--异常通知["+instance.getIdentity()+"]";
		String htmlContent = "<h2>监控ID："+instance.getIdentity()+"</h2>"
				+ "<h3>异常内容："+response+"</h3>";
		if(instance instanceof ClientInstance) {
			htmlContent += this.formatHtml((ClientInstance)instance);
		}else if(instance instanceof ServerInstance) {
			htmlContent += this.formatHtml((ServerInstance)instance);
		}else {
			try {
				htmlContent += "<p>"+JSONObject.toJSONString(instance)+"</p>";	
			}catch (Exception e) {
				htmlContent += "<span style=\"color:red;\">对象无法序列化打印，instance.toString() ==>> "+instance.toString()+"</span>";
			}
		}
		return new String[] {subject, htmlContent};
	}

	private String formatErrorWeixinContent(Instance instance, String response) {
		return "异常-项目可用监控\n"
				+ "  CODE["+instance.getIdentity()+"]对应的服务出现异常\n"
				+ "  异常内容："+response;
	}
}
