package com.taover.bazhuayun.analysis.web.permission;

import java.io.IOException;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.alibaba.fastjson.JSONObject;
import com.taover.util.UtilLog;
import com.taover.util.bean.UtilResultInfo;

public class JwtFilter implements Filter {
	private int adminUserId;
	
	public JwtFilter(int userId) {
		this.adminUserId = userId;
	}
	
    @Override
    public void doFilter(final ServletRequest req, final ServletResponse res, final FilterChain chain) throws IOException, ServletException {
        final HttpServletRequest request = (HttpServletRequest) req;
        final HttpServletResponse response = (HttpServletResponse) res;
    	if ("OPTIONS".equals(request.getMethod())) {
            response.setStatus(HttpServletResponse.SC_OK);
        } else {
            //切换为当前用户
        	String authHeader = request.getHeader("authorization");
        	if(authHeader == null || !authHeader.startsWith(JwtUtil.TOKEN_PREFFIX)) {
        		response.getWriter().write(JSONObject.toJSONString(UtilResultInfo.getNotAuthorized("")));
        		return;
        	}else {
        		try {
            		//检查是否有权限访问
            		BzyClaims claims = JwtUtil.parseJWT(authHeader.substring(JwtUtil.TOKEN_START_INDEX));
            		if(claims.getUserId().intValue() != adminUserId) {
            			response.getWriter().write(JSONObject.toJSONString(UtilResultInfo.getNotAuthorized("not admin,no permission")));
                		return;		
            		}
        		}catch (Exception e) {
        			response.addHeader("Content-Type", "application/json;charset=UTF-8");
        			response.getWriter().write(JSONObject.toJSONString(UtilResultInfo.getNotAuthorized(e.getMessage())));
        			return;
				}
        	}            
        }
        chain.doFilter(req, res);
    }

	@Override
	public void init(FilterConfig filterConfig) throws ServletException {
		UtilLog.infoForMessage("JwtFilter:init", this.getClass());
	}

	@Override
	public void destroy() {
		UtilLog.infoForMessage("JwtFilter:destroy", this.getClass());
	}
}
